/*
 * Date: Jan 10, 2004
 * Copyright (c) 2004 Matvey Kazakov.
 *
 * $Id$
 */
package ru.ifmo.ips.visualizers.backpack;

import javax.swing.*;
import java.awt.*;
import java.util.Iterator;
import java.text.MessageFormat;

/**
 *       
 * 
 * @author Matvey Kazakov
 */
public class Drawer extends JPanel {

    // ===============    ========================
    private static final Color COLOR_BORDER = Color.black;
    private static final Color COLOR_CURRENT = Color.lightGray;
    private static final Color COLOR_SELECT = Color.gray;
    private static final Color COLOR_BACKGROUND = Color.white;

    private static final Font plainFont = new Font("Arial", Font.PLAIN, 11);
    private static final Font headerFont = plainFont.deriveFont(Font.BOLD);

    // ==============   ============================
    private static final String MESSAGE_STATE_0 = " 0:  ";
    private static final String MESSAGE_STATE_1 = " 1:  {0}       ";
    private static final String MESSAGE_STATE_2 = " 2:  0      {0} ";
    private static final String MESSAGE_STATE_4 = " 4 ( ):     {0}, {1}:  ";
    private static final String MESSAGE_STATE_4_ALT = " 4 ( ):     {0}, {1} : ";
    private static final String MESSAGE_STATE_5_OK = " 5:     ";
    private static final String MESSAGE_STATE_5_FAIL = " 5:      ";
    private static final String MESSAGE_STATE_7_ALT = " 7:   : ";
    private static final String MESSAGE_STATE_7 = " 7:   ";
    private static final String MESSAGE_STATE_8 = " 8:   ";
    private static final String MESSAGE_STATE_9 = " 9: .    ";
    private static final String MESSAGE_STATE_10 = " 10: .   : {0}";

    private Globals globals;

    private JLabel messageLabel;
    private JLabel[][] T;
    private JLabel[] M;

    //   .      
    private JLabel previous = null;
    private JLabel prev_sel1 = null;
    private JLabel prev_sel2 = null;

    public Drawer(Globals globals) {
        this.globals = globals;
        setLayout(new BorderLayout());
        JPanel picturePanel = new JPanel();
        JPanel messagePanel = new JPanel(new BorderLayout());
        add(picturePanel, BorderLayout.CENTER);
        add(messagePanel, BorderLayout.SOUTH);
        messageLabel = new JLabel();
        messagePanel.add(messageLabel, BorderLayout.CENTER);
        //      .
        //      .
        T = new JLabel[globals.T.length][globals.T.length > 0 ? globals.T[0].length : 0];
        M = new JLabel[globals.K];
        picturePanel.setLayout(new GridLayout(globals.K + 2, globals.N + 4));
        picturePanel.setBackground(COLOR_BACKGROUND);
        picturePanel.setBorder(BorderFactory.createLineBorder(Color.white, 10));
        for (int i = 0; i < globals.K + 2; i++) {
            if (i > 0) {
                JLabel[] row = T[i - 1];
                for (int j = 0; j < globals.N + 4; j++) {
                    if (j > 2) {
                        JLabel cell = new JLabel();
                        row[j - 3] = cell;
                        picturePanel.add(createCell(cell));
                    } else if (j == 2) {
                        picturePanel.add(createHeaderCell(i - 1));
                    } else if (j == 0 && i > 1) {
                        M[i - 2] = new JLabel("" + globals.M[i - 2]);
                        picturePanel.add(createCell(M[i - 2]));
                    } else {
                        picturePanel.add(createSpacer());
                    }
                }
            } else {
                for (int j = 0; j < globals.N + 4; j++) {
                    if (j < 3) {
                        picturePanel.add(createSpacer());
                    } else {
                        picturePanel.add(createHeaderCell(j - 3));
                    }
                }
            }
        }
    }

    /**
     *     
     */
    private JPanel createCell(JLabel label) {
        JPanel panel = new JPanel(new BorderLayout());
        panel.setBorder(BorderFactory.createLineBorder(COLOR_BORDER, 1));
        panel.add(label, BorderLayout.CENTER);
        panel.setOpaque(false);
        label.setOpaque(false);
        label.setFont(plainFont);
        label.setHorizontalAlignment(SwingConstants.CENTER);
        return panel;
    }

    /**
     *    .
     */
    private JPanel createHeaderCell(int i) {
        JLabel label = new JLabel("" + i);
        JPanel panel = createCell(label);
        label.setFont(headerFont);
        return panel;
    }

    /**
     *   
     */
    private JPanel createSpacer() {
        JPanel panel = new JPanel();
        panel.setOpaque(false);
        return panel;
    }

    /**
     *  
     */
    public void redraw() {
        //   
        for (int i = 0; i < globals.K + 1; i++) {
            for (int j = 0; j < globals.N + 1; j++) {
                T[i][j].setText(globals.T[i][j] != -1 ? "" + globals.T[i][j] : "");
            }
        }
        for (int i = 0; i < globals.K; i++) {
            if (globals.positions.contains(new Integer(i))) {
                selectCell(M[i], COLOR_CURRENT);
            } else {
                deselectCell(M[i]);
            }
        }
        //     
        deselectCell(previous);
        deselectCell(prev_sel1);
        deselectCell(prev_sel2);
        String newMessage = null;
        //          
        switch (globals.state) {
            case 0:
                newMessage = MESSAGE_STATE_0;
                break;
            case 1:
                newMessage = MessageFormat.format(MESSAGE_STATE_1, new Object[]{new Integer(globals.j)});
                selectCell(previous = T[globals.i][globals.j], COLOR_CURRENT);
                break;
            case 2:
                newMessage = MessageFormat.format(MESSAGE_STATE_2, new Object[]{new Integer(globals.i)});
                selectCell(previous = T[globals.i][globals.j], COLOR_CURRENT);
                break;
            case 4:
                if (globals.j >= globals.M[globals.i - 1]) {
                    newMessage = MessageFormat.format(MESSAGE_STATE_4_ALT, new Object[]{
                        new Integer(globals.i), new Integer(globals.j)
                    });
                } else {
                    newMessage = MessageFormat.format(MESSAGE_STATE_4, new Object[]{
                        new Integer(globals.i), new Integer(globals.j)
                    });
                }
                selectCell(previous = T[globals.i][globals.j], COLOR_CURRENT);
                selectCell(prev_sel1 = T[globals.i - 1][globals.j], COLOR_SELECT);
                if (globals.j > globals.M[globals.i - 1]) {
                    selectCell(prev_sel2 = T[globals.i - 1][globals.j - globals.M[globals.i - 1]], COLOR_SELECT);
                }
                break;
            case 5:
                if (globals.T[globals.K][globals.N] == 1) {
                    newMessage = MESSAGE_STATE_5_OK;
                } else {
                    newMessage = MESSAGE_STATE_5_FAIL;
                }
                selectCell(previous = T[globals.i - 1][globals.sum], COLOR_CURRENT);
                break;
            case 7:
                if (globals.j >= globals.M[globals.i - 1]) {
                    newMessage = MESSAGE_STATE_7_ALT;
                } else {
                    newMessage = MESSAGE_STATE_7;
                }
                selectCell(previous = T[globals.i][globals.sum], COLOR_CURRENT);
                selectCell(prev_sel1 = T[globals.i - 1][globals.sum], COLOR_SELECT);
                if (globals.sum >= globals.M[globals.i - 1]) {
                    selectCell(prev_sel2 = T[globals.i - 1][globals.sum - globals.M[globals.i - 1]], COLOR_SELECT);
                }
                break;
            case 8:
                newMessage = MESSAGE_STATE_8;
                selectCell(previous = T[globals.i - 1][globals.sum], COLOR_CURRENT);
                break;
            case 9:
                newMessage = MESSAGE_STATE_9;
                break;
            case 10:
                String itemsString = "[";
                boolean first = true;
                for (Iterator iterator = globals.positions.iterator(); iterator.hasNext();) {
                    Integer pos = (Integer) iterator.next();
                    if (first) {
                        first = false;
                    } else {
                        itemsString += ", ";
                    }
                    itemsString += globals.M[pos.intValue()];
                }
                itemsString += "]";
                newMessage = MessageFormat.format(MESSAGE_STATE_10 , new Object[]{itemsString});
                break;
        }
        messageLabel.setText(newMessage);
    }

    /**
     *    
     */
    private void selectCell(JLabel cell, Color color) {
        cell.setOpaque(true);
        cell.setBackground(color);
        cell.revalidate();
        cell.repaint();
    }

    private void deselectCell(JLabel cell) {
        if (cell != null) {
            cell.setOpaque(false);
            cell.revalidate();
            cell.repaint();
        }
    }

}

/*
 * $Log$
 */