/*
 * Date: Jan 10, 2004
 * Copyright (c) 2004 Matvey Kazakov.
 *
 * $Id$
 */
package ru.ifmo.ips.visualizers.backpack;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.text.MessageFormat;

/**
 *   
 * 
 * @author Matvey Kazakov
 */
public class Visio extends JPanel implements ActionListener {

    //   
    private static final String LABEL_AUTO = "";
    private static final String LABEL_STOP = "";
    private static final String LABEL_STEP = ">>";
    private static final String LABEL_RESTART = "";
    private static final String LABEL_DELAYUP = ">";
    private static final String LABEL_DELAYDOWN = "<";
    private static final String LABEL_DELAY = ": {0}";

    // 
    private JButton buttonStep = new JButton(LABEL_STEP);
    private JButton buttonRestart = new JButton(LABEL_RESTART);
    private JButton buttonAuto = new JButton(LABEL_AUTO);
    private JButton buttonDelayDown = new JButton(LABEL_DELAYDOWN);
    private JButton buttonDelayUp = new JButton(LABEL_DELAYUP);
    private JLabel labelDelay = new JLabel();

    // 
    private static final int INITIAL_DELAY = 4;
    private static final int[] DELAYS = new int[]{10, 20, 50, 100, 250, 500, 1000};

    //  
    private Globals globals = new Globals(new int[]{4, 5, 3, 7, 6}, 16);
    // 
    private Automaton automaton = new Automaton(globals);
    //    
    private Drawer drawer = new Drawer(globals);

    private int delay = INITIAL_DELAY;
    private boolean auto = false;

    // ,    
    private Thread autoThread = new AutoThread();

    public static void main(String[] args) {
        JFrame frame = new JFrame("  ");
        Visio visio = new Visio();
        visio.start();
        frame.setContentPane(visio);
        frame.pack();
        frame.setLocationRelativeTo(null);
        frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        frame.show();
    }

    /**
     *   
     */
    public Visio() {
        setBorder(BorderFactory.createLineBorder(getBackground(), 5));
        setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
        JPanel buttonPanel = new JPanel();
        buttonPanel.setLayout(new BoxLayout(buttonPanel, BoxLayout.X_AXIS));
        add(drawer);
        buttonStep.addActionListener(this);
        buttonRestart.addActionListener(this);
        buttonAuto.addActionListener(this);
        buttonDelayDown.addActionListener(this);
        buttonDelayUp.addActionListener(this);

        buttonPanel.add(buttonStep);
        buttonPanel.add(Box.createHorizontalStrut(10));
        buttonPanel.add(buttonRestart);
        buttonPanel.add(Box.createHorizontalStrut(10));
        buttonPanel.add(Box.createHorizontalGlue());

        buttonPanel.add(buttonAuto);
        buttonPanel.add(Box.createHorizontalStrut(10));
        buttonDelayDown.setMargin(new Insets(0, 3, 0, 3));
        buttonDelayUp.setMargin(new Insets(0, 3, 0, 3));
        buttonPanel.add(buttonDelayDown);
        buttonPanel.add(Box.createHorizontalStrut(5));
        buttonPanel.add(labelDelay);
        buttonPanel.add(Box.createHorizontalStrut(5));
        buttonPanel.add(buttonDelayUp);

        updateLabelDelay();

        add(Box.createVerticalStrut(10));
        add(buttonPanel);
        drawer.redraw();
    }

    /**
     *    
     */
    public void start() {
        autoThread.start();
    }

    /**
     *    
     */
    public void stop() {
        autoThread.interrupt();
    }
    
    /**
     *    
     */
    public void actionPerformed(ActionEvent e) {
        Object source = e.getSource();
        if (source == buttonStep) {
            automaton.makeStep();
            drawer.redraw();
            updateButtonStep();
            updateAutoButton();
        } else if (source == buttonRestart) {
            globals.init();
            auto = false;
            updateAutoButton();
            drawer.redraw();
            updateButtonStep();
        } else if (source == buttonAuto) {
            auto = !auto;
            updateAutoButton();
        } else if (source == buttonDelayDown) {
            if (delay > 0) {
                delay--;
                updateLabelDelay();
                autoThread.interrupt();
            }
            updateDelayButtonsState();
        } else if (source == buttonDelayUp) {
            if (delay < DELAYS.length) {
                delay++;
                updateLabelDelay();
                autoThread.interrupt();
            }
            updateDelayButtonsState();
        }
    }

    // ==========     =====================

    private void updateButtonStep() {
        buttonStep.setEnabled(!automaton.isFinished());
    }

    private void updateDelayButtonsState() {
        buttonDelayDown.setEnabled(delay > 0);
        buttonDelayUp.setEnabled(delay < DELAYS.length);
    }

    private void updateLabelDelay() {
        labelDelay.setText(MessageFormat.format(LABEL_DELAY, new Object[]{"" + DELAYS[delay]}));
    }

    private void updateAutoButton() {
        if (automaton.isFinished()) {
            auto = false;
        }
        buttonAuto.setText(auto ? LABEL_STOP : LABEL_AUTO);
    }

    /**
     * ,        
     *  .
     */
    private class AutoThread extends Thread {
        public AutoThread() {
            super("Auto thread");
            setDaemon(true);
        }

        public void run() {
            while (true) {
                if (auto) {
                    actionPerformed(new ActionEvent(buttonStep, 0, ""));
                }
                try {
                    sleep(DELAYS[delay]);
                } catch (InterruptedException e) {
                }
            }
        }
    }

}

/*
 * $Log$
 */